from unittest import TestCase

from .context import Image


class ImageTest(TestCase):

    def test_name(self):
        image = Image(nom='test')
        self.assertEqual('test', image.nom, "Name should be test")
        image.nom = 'override'
        self.assertEqual('override', image.nom, "Name should be override")
        image.nom = None
        self.assertIsNone(image.nom, "Name should be None")

    def test_tags(self):
        image = Image(tags=['voiture', 'vacance'])
        self.assertEqual(['voiture', 'vacance'], image.tags, "Tags should be voiture and vacance")
        image.tags = ['soleil']
        self.assertEqual(['soleil'], image.tags, "Tags should be soleil")
        image.tags = None
        self.assertIsNone(image.tags, "Tags should be None")

    def test_none_tags(self):
        image = Image()
        self.assertEqual([], image.tags, "Tags should be empty")

    def test_operations(self):
        image = Image(operations=['tagger', 'resizer'])
        self.assertEqual(['tagger', 'resizer'], image.operations, "Tags should be tagger and resizer")
        image.operations = ['tagger']
        self.assertEqual(['tagger'], image.operations, "Tags should be tagger")
        image.operations = None
        self.assertIsNone(image.operations, "Tags should be None")

    def test_file(self):
        file = open("resources/image.jpeg", "rb")
        image = Image(file=file)
        self.assertEqual(file, image.file, "Le fichier ne correspond pas lors de l'initialisation")
        temp = open("resources/temp.jpeg", "rb")
        image.file = temp
        self.assertEqual(temp, image.file, "Le fichier ne correspond pas lors du setter")
        image.file = None
        self.assertIsNone(image.file, "File should be None")



from datetime import date

import tornado
from logging_utils import Chronometer, LogDecorator
from tornado.web import MissingArgumentError

from .image import Image
from .rabbit import Messaging

messenger = Messaging()


class UploadHandler(tornado.web.RequestHandler):

    def data_received(self, chunk):
        pass

    @Chronometer(function_name='upload-post-timer')
    @LogDecorator(decorator_log='upload-post-inspect')
    def post(self):  # POST /upload

        try:
            nom = self.get_argument('nom')
        except MissingArgumentError:
            nom = None

        try:
            tags = self.get_argument('tags')
            if tags:
                tags = tags.split(',')
            else:
                tags = []
        except MissingArgumentError:
            tags = []

        try:
            operations = self.get_argument('operations')
            if operations:
                operations = operations.split(',')
            else:
                operations = []
        except MissingArgumentError:
            operations = []

        description = self.get_argument('description')
        date_create = date.today()
        categorie = self.get_argument('categorie')

        print(f"nombre d'images {len(self.request.files)}")
        for _, files in self.request.files.items():
            for info in files:
                filename = info["filename"]
                body = info["body"]

                output_file = open("upload/" + filename, "wb")
                output_file.write(body)

                input_file = open("upload/" + filename, "rb")

                image = Image(
                    description=description,
                    categorie=categorie,
                    tags=tags,
                    date=date_create,
                    nom=nom,
                    file=input_file,
                    operations=operations)

                if image.validate():
                    messenger.send(image)
                    self.finish({'image': nom, 'operations': operations})
                else:
                    self.set_status(400)
                    self.finish({'error': 'bad request', 'reason': 'consumers are not valid'})



from logging_utils import LogDecorator

from core.consumer.api import Consumer


class Image:

    def __init__(self,
                 description=None,
                 categorie=None,
                 tags=None,
                 date=None,
                 date_update=None,
                 nom=None,
                 file=None,
                 operations=None):

        """
        Constructeur de la classe Image
        :param description: Description de l'image
        :param categorie: Catégorie de l'image
        :param tags: Tags de l'image
        :param date: Date de création de l'image
        :param date_update: Date de mise à jour de l'image
        :param nom: Nom de l'imagePOST
        :param file: Fichier image
        """
        self.__description = description
        self.__categorie = categorie
        self.__tags = tags
        self.__date = date
        self.__date_update = date_update
        self.__nom = nom
        self.__file = file
        self.__operations = operations

        if not self.__operations:
            self.__operations = []

        if not self.__tags:
            self.__tags = []

    @property
    def description(self):
        return self.__description

    @description.setter
    def description(self, value):
        self.__description = value

    @property
    def nom(self):
        return self.__nom

    @nom.setter
    def nom(self, value):
        self.__nom = value

    @property
    def file(self):
        return self.__file

    @file.setter
    def file(self, value):
        self.__file = value

    @property
    def tags(self) -> list:
        return self.__tags

    @tags.setter
    def tags(self, tags: list):
        self.__tags = tags

    @property
    def date_create(self):
        return self.__date

    @date_create.setter
    def date_create(self, value):
        self.__date = value

    @property
    def date_update(self):
        return self.__date_update

    @date_update.setter
    def date_update(self, value):
        self.__date_update = value

    @property
    def categorie(self):
        return self.__categorie

    @categorie.setter
    def categorie(self, value):
        self.__categorie = value

    @property
    def operations(self) -> list:
        return self.__operations

    @operations.setter
    def operations(self, value: list):
        self.__operations = value

    @LogDecorator('imag-validate-inspect')
    def validate(self):
        consumers = [Consumer.find_by_name(operation) for operation in self.__operations]

        if any(consumer is None for consumer in consumers):
            print("Error on unknown consumer")
            return False

        if sum(consumer.final for consumer in consumers) != 1:
            print("Error on non-final consumer")
            return False

        return True




import coloredlogs
import tornado

from core.cm.commandLine import Application

coloredlogs.install(level='DEBUG')

sockets = tornado.netutil.bind_sockets(8888, '')
server = tornado.httpserver.HTTPServer(Application())
server.add_sockets(sockets)

for s in sockets:
    print('Listening on %s, port %d' % s.getsockname()[:2])

tornado.ioloop.IOLoop.instance().start()


import uuid
import time
from threading import Thread

import tornado
from tornado.web import RequestHandler
from uuid import uuid5

from logging_utils import Chronometer, LogDecorator

from core.loader.rabbit import Messaging


class Consumer:

    objects = []

    @staticmethod
    def find_by_oid(oid):
        try:
            return next(item for item in Consumer.objects if item.oid == oid)
        except StopIteration:
            return None

    @staticmethod
    def find_by_name(name):
        try:
            return next(item for item in Consumer.objects if item.name == name)
        except StopIteration:
            return None

    def __init__(self, name, heartbeat_url, final=False):
        self.__oid = str(uuid5(uuid.NAMESPACE_OID, name))
        self.__name = name
        self.__final = final
        self.__heartbeat_url = heartbeat_url
        self.__class__.objects.append(self)
        self.__alive = True

    @LogDecorator('consumer_del_inspect')
    def __del__(self):
        self.__class__.objects.remove(self)

    def __dict__(self):
        return {'oid': self.__oid, 'name': self.__name, 'heartbeat_url': self.__heartbeat_url, 'final': self.__final,
                'status': 'UP' if self.__alive else 'DOWN'}

    @property
    def name(self):
        return self.__name

    @property
    def final(self):
        return self.__final

    @property
    def oid(self):
        return self.__oid

    @property
    def heartbeat_url(self):
        return self.__heartbeat_url

    @property
    def status(self):
        return 'UP' if self.__alive else 'DOWN'

    @status.setter
    def status(self, value):
        self.__alive = value


class ConsumerHandler(RequestHandler):

    def data_received(self, chunk):
        pass

    @Chronometer(function_name='consumer-post-timer')
    @LogDecorator(decorator_log='consumer-post-inspect')
    def post(self):  # POST /consumer

        consumer_json = tornado.escape.json_decode(self.request.body)

        if 'name' not in consumer_json or 'heartbeat_url' not in consumer_json:
            self.set_status(400)
            self.finish({'error': 'bad request', 'reason': "'name' and 'heartbeat_url' are mandatory."})

        if not Consumer.objects:
            consumer = Consumer(consumer_json['name'], consumer_json['heartbeat_url'],
                                consumer_json['final'] if 'final' in consumer_json else False)
        elif not any(consumer_json['name'] == consumer.name
                     and consumer_json['heartbeat_url'] == consumer.heartbeat_url for consumer in Consumer.objects):
            consumer = Consumer(consumer_json['name'], consumer_json['heartbeat_url'],
                                consumer_json['final'] if 'final' in consumer_json else False)
        else:
            self.set_status(409)
            self.finish({'error': 'conflict', 'reason': "Consumer already register"})

        self.set_status(200)
        self.finish({'consumer': consumer.name, 'register': True,
                     'informations': f"/admin/consumers/{consumer.oid}"})

    @Chronometer(function_name='consumer-delete-timer')
    @LogDecorator(decorator_log='consumer-delete-inspect')
    def delete(self, slug):  # DELETE /consumer/oid
        consumer = Consumer.find_by_oid(slug)

        if not consumer:
            self.set_status(404)
            self.finish({'error': 'not found', 'reason': f"Consumer identified by '{slug}' not exist"})
        else:
            consumer.__del__()
            self.set_status(202)
            self.finish({'action': 'delete', 'msg': f"Consumer identified by '{slug}' not exist anymore"})


class AdminConsumerHandler(RequestHandler):

    def data_received(self, chunk):
        pass

    @Chronometer(function_name='resource-post-timer')
    @LogDecorator(decorator_log='resource-post-inspect')
    def get(self, slug):  # GET /admin/consumers/_slug_
        if slug == 'list':
            self.set_status(200)
            self.finish({'nb_consumers_total': len(Consumer.objects),
                         'consumers': [item.__dict__() for item in Consumer.objects]})
        else:
            consumer = Consumer.find_by_oid(slug)
            if not consumer:
                self.set_status(404)
                self.finish({'error': 'not found', 'reason': "no consumers exists"})
            else:
                self.set_status(200)
                self.finish(consumer.__dict__())


@Chronometer('heartbeat-timer')
def background_task():
    print("Start heartbeat")
    for consumer in Consumer.objects:
        count = Messaging.get_consumers(consumer.name)
        consumer.status = count == 0
    print("Waiting 30 seconds")
    time.sleep(30)


t = Thread(target=background_task)
t.start()



import os
import random

import pika
from logging_utils import Chronometer, LogDecorator

RABBIT_HOST = os.getenv('RABBIT_HOST', '127.0.0.1')


class Messaging:

    @staticmethod
    def __create_connection():
        return pika.BlockingConnection(pika.ConnectionParameters(
            host=RABBIT_HOST
        ))

    @Chronometer(function_name='messaging-send-timer')
    @LogDecorator(decorator_log='messaging-send-inspect')
    def send(self, image):

        if image.operations:
            random.shuffle(image.operations)
        else:
            image.operations.append('to_save')

        queue_name = image.operations.pop()

        connexion = Messaging.__create_connection()

        channel = connexion.channel()
        channel.queue_declare(queue=queue_name)

        properties = {'categorie': image.categorie, 'tags': image.tags, 'description': image.description,
                      'nom': image.nom, 'operations': image.operations}

        channel.basic_publish(
            exchange='',
            routing_key=queue_name,
            body=image.file.read(),
            properties=pika.BasicProperties(headers=properties)
        )

        print(f'[x] image sent to {queue_name}')

        connexion.close()

        return queue_name

    @staticmethod
    def get_consumers(queue_name):
        connexion = Messaging.__create_connection()

        channel = connexion.channel()
        queue_state = channel.queue_declare(queue=queue_name)

        return queue_state.method.consumer_count



import click
import coloredlogs
import tornado

from ..consumer import AdminConsumerHandler, ConsumerHandler
from ..loader.upload import UploadHandler


class Application(tornado.web.Application):

    def __init__(self):
        handlers = [
            (r'/upload', UploadHandler),   # Point d'accès des uploads
            (r'/consumer', ConsumerHandler),  # Point d'accès des consumers
            (r'/consumer/([^/]+)', ConsumerHandler),
            (r'/admin/consumers/([^/]+)', AdminConsumerHandler)  # Point d'accès de l'admin consumers
        ]
        tornado.web.Application.__init__(self, handlers)


@click.command(name='core_server')
@click.option('--port', '-p', default=8888, help='Port listening server')
@click.option('--log-level', '-l', default='INFO', help='Log level')
def cm(port, log_level):
    """
    Utilitaire de ligne de commande permettant de récupérer tous les scénarii d'un test.
    :return:
    """
    coloredlogs.install(level=log_level)
    http_server = tornado.httpserver.HTTPServer(Application())
    http_server.listen(port)
    tornado.ioloop.IOLoop.instance().start()
